@@ -39,7 +39,6 @@ import (
 	_ "golang.org/x/sys/unix"
 	_ "golang.zx2c4.com/wireguard/device"
 	_ "golang.zx2c4.com/wireguard/tun"
-	_ "inet.af/netaddr"
 	_ "tailscale.com/hostinfo"
 	_ "tailscale.com/ipn"
 	_ "tailscale.com/ipn/ipnlocal"
@@ -48,6 +47,7 @@ import (
 	_ "tailscale.com/logtail"
 	_ "tailscale.com/logtail/filch"
 	_ "tailscale.com/net/dns"
+	_ "tailscale.com/net/netaddr"
 	_ "tailscale.com/net/tsdial"
 	_ "tailscale.com/net/tstun"
 	_ "tailscale.com/paths"
